using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Peak.Can.Basic;
using MyEventIO;

namespace PCAN_BasicExampleCE
{
    public partial class Form1 : Form
    {
        private byte hBus;
        private ushort language;
        private System.Threading.Thread hThread;
        private delegate void ReadDelegateHandler();
        private ReadDelegateHandler hReadDelegate;
        IntPtr hEvent;
        private volatile bool ExitThread;

        public Form1()
        {
            InitializeComponent();

            HwCombo.SelectedIndex = 0;
            BaudCombo.SelectedIndex = 0;

            ParamCombo.SelectedIndex = 0;
            OnOffCombo.SelectedIndex = 0;

            hBus = PCANBasic.PCAN_NONEBUS;

            hReadDelegate = new ReadDelegateHandler(ReadCANMessage);

            ExitThread = false;
            hEvent = EventIO.CreateEvent(IntPtr.Zero, false, false, "");
            
            //language = 7; // deutsch
            language = 9; // english
            //language = 10; // spanish
            //language = 12; // french
        }

        // build a small error info string and show it on a label
        private void ShowErrorText(string str, TPCANStatus err)
        {
            StringBuilder sb;

            sb = new StringBuilder(256);
            PCANBasic.GetErrorText(err, language, sb);

            InfoLabel.Text = str + ":\n[" + err.ToString() + ", " + sb.ToString() + "]\n";
        }

        private void ReadCANMessage()
        {
            TPCANStatus res;
            TPCANMsg msgbuff;
            string str = "";


            // read receive queue
            res = PCANBasic.Read(hBus, out msgbuff);

            ShowErrorText("Read", res);

            // check if message was read
            if (res == TPCANStatus.PCAN_ERROR_OK)
            {
                // check for status message
                if ((msgbuff.MSGTYPE & TPCANMessageType.PCAN_MESSAGE_STATUS) != 0)
                {
                    TPCANStatus stat;


                    // rebuild status from data byte 2 and 3
                    stat = (TPCANStatus)((uint)msgbuff.DATA[2] << 8 | (uint)msgbuff.DATA[3]);
                    str = "Status message: " + stat.ToString(); ;
                }

                else
                {
                    // message is data message, check 11/29 bit ID
                    if ((msgbuff.MSGTYPE & TPCANMessageType.PCAN_MESSAGE_EXTENDED) != 0)
                        str = string.Format("{0:X8}h  {1:D2}", msgbuff.ID, msgbuff.LEN);
                    else
                        str = string.Format("     {0:X3}h  {1:D2}", msgbuff.ID, msgbuff.LEN);


                    // check if message is remote request or data message
                    if ((msgbuff.MSGTYPE & TPCANMessageType.PCAN_MESSAGE_RTR) != 0)
                    {
                        str += "  RTR";
                    }

                    else
                    {
                        for (uint idx = 0; idx < msgbuff.LEN; idx++)
                        {
                            str += "  " + msgbuff.DATA[idx].ToString("X");
                        }
                    }
                }
            }

            InfoLabel.Text += str;
        }

        // Initialize the Channel
        private void button1_Click(object sender, EventArgs e)
        {
            TPCANStatus res;
            TPCANBaudrate baud;
            uint port;
            ushort irq;
            

            // default baudrate
            baud = TPCANBaudrate.PCAN_BAUD_500K;

            // select a baudrate
            switch (BaudCombo.SelectedIndex)
            {
                case 0: baud = TPCANBaudrate.PCAN_BAUD_1M; break;
                case 1: baud = TPCANBaudrate.PCAN_BAUD_500K; break;
                case 2: baud = TPCANBaudrate.PCAN_BAUD_250K; break;
                case 3: baud = TPCANBaudrate.PCAN_BAUD_125K; break;
                case 4: baud = TPCANBaudrate.PCAN_BAUD_100K; break;
                case 5: baud = TPCANBaudrate.PCAN_BAUD_50K; break;
                case 6: baud = TPCANBaudrate.PCAN_BAUD_20K; break;
                case 7: baud = TPCANBaudrate.PCAN_BAUD_10K; break;
                case 8: baud = TPCANBaudrate.PCAN_BAUD_5K; break;
            }

            // for ISA, get I/O Address
            port = uint.Parse(PortEdit.Text, System.Globalization.NumberStyles.HexNumber);

            // for ISA, get IRQ
            irq = ushort.Parse(IrqEdit.Text, System.Globalization.NumberStyles.Integer);


            // initialize the Channel, check if hBus is an ISA-Bus
            if (PCANBasic.PCAN_ISABUS1 <= hBus && hBus <= PCANBasic.PCAN_ISABUS8)
                res = PCANBasic.Initialize(hBus, baud, TPCANType.PCAN_TYPE_ISA_SJA, port, irq);
            else
                res = PCANBasic.Initialize(hBus, baud); // PnP device (PCI or USB)


            ShowErrorText("Initialize",res);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            TPCANStatus res;

            // Uninitialize the Channel
            res = PCANBasic.Uninitialize(hBus);

            ShowErrorText("Uninitialize",res);
        }

        private void button3_Click(object sender, EventArgs e)
        {
            TPCANStatus res;

            // reset the channel
            res = PCANBasic.Reset(hBus);

            ShowErrorText("Reset",res);
        }

        private void button4_Click(object sender, EventArgs e)
        {
            TPCANStatus res;

            // show status
            res = PCANBasic.GetStatus(hBus);

            ShowErrorText("GetStatus",res);
        }

        private void button5_Click(object sender, EventArgs e)
        {
            ReadCANMessage();
        }

        private void button6_Click(object sender, EventArgs e)
        {
            TPCANStatus res;
            TPCANMsg msgbuff;

            msgbuff = new TPCANMsg();

            msgbuff.DATA = new byte[8]; 
            
            // set ID and LEN
            msgbuff.ID = uint.Parse(TxIdEdit.Text, System.Globalization.NumberStyles.HexNumber);
            msgbuff.LEN = byte.Parse(TxLenEdit.Text, System.Globalization.NumberStyles.Integer);

            // select message format 11/29 bit ID
            if (ExtCheckBox.Checked)
                msgbuff.MSGTYPE = TPCANMessageType.PCAN_MESSAGE_EXTENDED;
            else
                msgbuff.MSGTYPE = TPCANMessageType.PCAN_MESSAGE_STANDARD;


            // set RTR flag if remote request frame
            if (RTRCheckBox.Checked)
            {
                msgbuff.MSGTYPE |= TPCANMessageType.PCAN_MESSAGE_RTR;
            }
            else
            {
                // set data bytes
                msgbuff.DATA[0] = byte.Parse(TxD0Edit.Text, System.Globalization.NumberStyles.HexNumber);
                msgbuff.DATA[1] = byte.Parse(TxD1Edit.Text, System.Globalization.NumberStyles.HexNumber);
                msgbuff.DATA[2] = byte.Parse(TxD2Edit.Text, System.Globalization.NumberStyles.HexNumber);
                msgbuff.DATA[3] = byte.Parse(TxD3Edit.Text, System.Globalization.NumberStyles.HexNumber);
                msgbuff.DATA[4] = byte.Parse(TxD4Edit.Text, System.Globalization.NumberStyles.HexNumber);
                msgbuff.DATA[5] = byte.Parse(TxD5Edit.Text, System.Globalization.NumberStyles.HexNumber);
                msgbuff.DATA[6] = byte.Parse(TxD6Edit.Text, System.Globalization.NumberStyles.HexNumber);
                msgbuff.DATA[7] = byte.Parse(TxD7Edit.Text, System.Globalization.NumberStyles.HexNumber);
            }

            // send message
            res = PCANBasic.Write(hBus, ref msgbuff);

            ShowErrorText("Write",res);
        }

        private void button7_Click(object sender, EventArgs e)
        {
            TPCANStatus res;
            uint FromID, ToID;
            TPCANMode mode;


            // check format
            if (ExtFilterCheck.Checked)
                mode = TPCANMode.PCAN_MODE_EXTENDED;
            else
                mode = TPCANMode.PCAN_MODE_STANDARD;


            // parse the ID values
            FromID = uint.Parse(FromIDEdit.Text, System.Globalization.NumberStyles.HexNumber);
            ToID = uint.Parse(ToIDEdit.Text, System.Globalization.NumberStyles.HexNumber);

            // extend the filter
            res = PCANBasic.FilterMessages(hBus, FromID, ToID, mode);

            ShowErrorText("FilterMessages",res);
        }

        private void button8_Click(object sender, EventArgs e)
        {
            TPCANStatus res;
            uint paramval = 0;
            StringBuilder sb;
            string str;

            sb = new StringBuilder(256);

            switch (ParamCombo.SelectedIndex)
            {
                case 0:
                    res = PCANBasic.GetValue(hBus, TPCANParameter.PCAN_DEVICE_NUMBER, out paramval, 4);

                    ShowErrorText("GetValue(PCAN_DEVICE_NUMBER)",res);
                    InfoLabel.Text += paramval.ToString();
                    break;

                case 1:
                    res = PCANBasic.GetValue(hBus, TPCANParameter.PCAN_MESSAGE_FILTER, out paramval, 4);

                    ShowErrorText("GetValue(PCAN_MESSAGE_FILTER)", res);

                    if (paramval == PCANBasic.PCAN_FILTER_CLOSE)
                        str = "PCAN_FILTER_CLOSE";

                    else if (paramval == PCANBasic.PCAN_FILTER_CUSTOM)
                        str = "PCAN_FILTER_CUSTOM";

                    else if (paramval == PCANBasic.PCAN_FILTER_OPEN)
                        str = "PCAN_FILTER_OPEN";

                    else
                        str = "undef!";

                    InfoLabel.Text += str;
                    break;

                case 2:
                    res = PCANBasic.GetValue(hBus, TPCANParameter.PCAN_API_VERSION, sb, (uint)sb.Capacity);

                    ShowErrorText("GetValue(PCAN_API_VERSION)", res);
                    InfoLabel.Text += sb.ToString();
                    break;

                case 3:
                    res = PCANBasic.GetValue(hBus, TPCANParameter.PCAN_CHANNEL_VERSION, sb, (uint)sb.Capacity);

                    ShowErrorText("GetValue(PCAN_CHANNEL_VERSION)", res);
                    InfoLabel.Text += sb.ToString();
                    break;

                case 4:
                    res = PCANBasic.GetValue(hBus, TPCANParameter.PCAN_BUSOFF_AUTORESET, out paramval, 4);

                    ShowErrorText("GetValue(PCAN_BUSOFF_AUTORESET)", res);

                    if (paramval == PCANBasic.PCAN_PARAMETER_ON)
                        str = "PCAN_PARAMETER_ON";

                    else if (paramval == PCANBasic.PCAN_PARAMETER_OFF)
                        str = "PCAN_PARAMETER_OFF";

                    else
                        str = "undef!";

                    InfoLabel.Text += str;
                    break;

                case 5:
                    res = PCANBasic.GetValue(hBus, TPCANParameter.PCAN_LISTEN_ONLY, out paramval, 4);

                    ShowErrorText("GetValue(PCAN_LISTEN_ONLY)", res); 

                    if (paramval == PCANBasic.PCAN_PARAMETER_ON)
                        str = "PCAN_PARAMETER_ON";

                    else if (paramval == PCANBasic.PCAN_PARAMETER_OFF)
                        str = "PCAN_PARAMETER_OFF";

                    else
                        str = "undef!";

                    InfoLabel.Text += str;
                    break;

                case 6:
                    res = PCANBasic.GetValue(hBus, TPCANParameter.PCAN_CHANNEL_CONDITION, out paramval, 4);

                    ShowErrorText("GetValue(PCAN_CHANNEL_CONDITION)", res);

                    if (paramval == PCANBasic.PCAN_CHANNEL_AVAILABLE)
                        str = "PCAN_CHANNEL_AVAILABLE";

                    else if (paramval == PCANBasic.PCAN_CHANNEL_UNAVAILABLE)
                        str = "PCAN_CHANNEL_UNAVAILABLE";

                    else if (paramval == PCANBasic.PCAN_CHANNEL_OCCUPIED)
                        str = "PCAN_CHANNEL_OCCUPIED";

                    else
                        str = "undef!";

                    InfoLabel.Text += str;
                    break;
            }
        }

        private void button9_Click(object sender, EventArgs e)
        {
            TPCANStatus res;
            uint paramval;

            paramval = PCANBasic.PCAN_FILTER_CLOSE;

            // close the filter, no more messages will be received
            res = PCANBasic.SetValue(hBus, TPCANParameter.PCAN_MESSAGE_FILTER, ref paramval, 4);

            ShowErrorText("SetValue(PCAN_MESSAGE_FILTER)", res);
        }

        private void button10_Click(object sender, EventArgs e)
        {
            TPCANStatus res;
            uint paramval = 0;
            StringBuilder sb;

            sb = new StringBuilder(256);

            switch (ParamCombo.SelectedIndex)
            {
                case 0:

                    paramval = uint.Parse(ValNumEdit.Text, System.Globalization.NumberStyles.HexNumber);

                    res = PCANBasic.SetValue(hBus, TPCANParameter.PCAN_DEVICE_NUMBER, ref paramval, 4);

                    ShowErrorText("SetValue(PCAN_DEVICE_NUMBER)", res);
                    break;


                case 4:

                    if (OnOffCombo.SelectedIndex == 0)
                        paramval = PCANBasic.PCAN_PARAMETER_OFF;

                    else
                        paramval = PCANBasic.PCAN_PARAMETER_ON;

                    res = PCANBasic.SetValue(hBus, TPCANParameter.PCAN_BUSOFF_AUTORESET, ref paramval, 4);

                    ShowErrorText("SetValue(PCAN_BUSOFF_AUTORESET)", res);
                    break;


                case 5:

                    if (OnOffCombo.SelectedIndex == 0)
                        paramval = PCANBasic.PCAN_PARAMETER_OFF;

                    else
                        paramval = PCANBasic.PCAN_PARAMETER_ON;

                    res = PCANBasic.SetValue(hBus, TPCANParameter.PCAN_LISTEN_ONLY, ref paramval, 4);

                    ShowErrorText("SetValue(PCAN_LISTEN_ONLY)", res);
                    break;
            }
        }

        private void HwCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            // set the selected channel
            switch (HwCombo.SelectedIndex)
            {
                case 0: hBus = PCANBasic.PCAN_ISABUS1; break;
                case 1: hBus = PCANBasic.PCAN_ISABUS2; break;
                case 2: hBus = PCANBasic.PCAN_ISABUS3; break;
                case 3: hBus = PCANBasic.PCAN_ISABUS4; break;
                case 4: hBus = PCANBasic.PCAN_ISABUS5; break;
                case 5: hBus = PCANBasic.PCAN_ISABUS6; break;
                case 6: hBus = PCANBasic.PCAN_ISABUS7; break;
                case 7: hBus = PCANBasic.PCAN_ISABUS8; break;
                case 8: hBus = PCANBasic.PCAN_PCIBUS1; break;
                case 9: hBus = PCANBasic.PCAN_PCIBUS2; break;
                case 10: hBus = PCANBasic.PCAN_PCIBUS3; break;
                case 11: hBus = PCANBasic.PCAN_PCIBUS4; break;
                case 12: hBus = PCANBasic.PCAN_PCIBUS5; break;
                case 13: hBus = PCANBasic.PCAN_PCIBUS6; break;
                case 14: hBus = PCANBasic.PCAN_PCIBUS7; break;
                case 15: hBus = PCANBasic.PCAN_PCIBUS8; break;
                case 16: hBus = PCANBasic.PCAN_USBBUS1; break;
                case 17: hBus = PCANBasic.PCAN_USBBUS2; break;
                case 18: hBus = PCANBasic.PCAN_USBBUS3; break;
                case 19: hBus = PCANBasic.PCAN_USBBUS4; break;
                case 20: hBus = PCANBasic.PCAN_USBBUS5; break;
                case 21: hBus = PCANBasic.PCAN_USBBUS6; break;
                case 22: hBus = PCANBasic.PCAN_USBBUS7; break;
                case 23: hBus = PCANBasic.PCAN_USBBUS8; break;
            }
        }

        private void CANReadThread()
        {
            TPCANStatus res;
            UInt32 paramval;


            paramval = Convert.ToUInt32(hEvent.ToInt32());
            res = PCANBasic.SetValue(hBus, TPCANParameter.PCAN_RECEIVE_EVENT, ref paramval, sizeof(uint));

            while (!ExitThread)
            {
                EventIO.WaitForSingleObject(hEvent, -1);

                // read receive queue
                this.Invoke(hReadDelegate);
            }
        }

        private void UseEventCheck_CheckStateChanged(object sender, EventArgs e)
        {
            if (hBus == PCANBasic.PCAN_NONEBUS)
                return;

            if (UseEventCheck.Checked)
            {
                // disable the Read button
                button5.Enabled = false;
                
                ExitThread = false;

                if (hThread == null)
                {
                    // start the thread
                    System.Threading.ThreadStart threadDelegate = new System.Threading.ThreadStart(this.CANReadThread);

                    hThread = new System.Threading.Thread(threadDelegate);
                    hThread.Start();
                }
            }

            else
            {
                if (hThread != null)
                {
                    // stop and release the thread
                    ExitThread = true;
                    EventIO.SetEvent(hEvent);
                    hThread.Abort();
                    hThread.Join();
                    hThread = null;
                }

                // enable the read button
                button5.Enabled = true;
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            UseEventCheck.Checked = false;
        }
    }
}